#ifndef FGGL_DATA_STORAGE_H
#define FGGL_DATA_STORAGE_H

#include <iostream>
#include <string>
#include <filesystem>
#include <vector>

namespace fggl::data {

	template<typename T>
	bool fggl_serialize(std::filesystem::path &data, const T *out);

	template<typename T>
	bool fggl_deserialize(std::filesystem::path &data, T *out);

	enum StorageType { Data, User, Cache };

	class Storage {

		public:
			template<typename T>
			bool load(StorageType pool, const std::string &name, T *out) {
				auto path = resolvePath(pool, name);
				if (!std::filesystem::exists(path)) {
					std::cerr << "Path " << path << " does not exist!" << std::endl;
					return false;
				}
				return fggl_deserialize<T>(path, out);
			}

			std::vector<std::filesystem::path> findResources(std::filesystem::path root, const std::string ext) {
				if ( !std::filesystem::exists(root) ) {
					return {};
				}

				std::vector<std::filesystem::path> paths;
				for ( const auto& entry : std::filesystem::recursive_directory_iterator(root) ) {
					if ( entry.is_regular_file() && entry.path().extension() == ext ) {
						paths.push_back( entry );
					}
				}
				return paths;
			}

			template<typename T>
			void save(StorageType pool, const std::string &name, const T *out) {
				auto path = resolvePath(pool, name);
				fggl_serialize<T>(path, out);
			}

			inline std::filesystem::path resolvePath(StorageType pool, const std::string &name) {
				std::filesystem::path path;

				switch (pool) {
				case Data: path = std::filesystem::current_path() / "data";
					break;
				case User: path = "./user-data/";
					break;
				case Cache: path = "/tmp/fggl/";
					break;
				}

				if (!std::filesystem::exists(path)) {
					std::filesystem::create_directories(path);
				}

				return path / name;
			}
	};

}

#endif
